/*
 * Copyright (c) 2000 Paul Herman
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef TCPSTAT_TCPSTAT_H
#define TCPSTAT_TCPSTAT_H

#ifdef HAVE_CONFIG_H
# include "config.h"
#else  /* Hmmm, try to wing it */
# ifdef __linux__
# define _BSD_SOURCE
# endif
#endif /* HAVE_CONFIG_H */

#include <stdio.h>		/* printf() */
#include <unistd.h>		/* fork(), execlp() */
#include <stdlib.h>		/* free(), malloc() */
#include <string.h>		/* bzero() */
#include <math.h>		/* sqrt() */
#include <ctype.h>		/* tolower */

#include <sys/time.h>		/* struct timeval */
#include <sys/wait.h>		/* waitpid() */

#include <sys/socket.h>		/* needed by INET stuff (sockaddr) */
#include <netinet/in.h>		/* needed by INET stuff (in_addr) */
#include <netinet/in_systm.h>	/* needed by INET stuff (n_long) */

#ifdef HAVE_NET_ETHERNET_H
#include <net/ethernet.h>	/* struct ether_header */
#else
#include "ethernet_stub.h"	/* Hope this works... */
#endif

#include <net/if_arp.h>		/* struct arp_hdr */
#include <netinet/ip.h>		/* struct ip */
#include <netinet/tcp.h>	/* struct tcphdr */
#include <netinet/udp.h>	/* struct udphdr */
#include <netinet/ip_icmp.h>	/* struct icmp */
#include <arpa/inet.h>		/* inet_ntoa() */
#include <netdb.h>		/* getprotobynumber */
#include <pcap.h>		/* pcap_*() */

#include <signal.h>

#ifdef HAVE_STRINGS_H
# include <strings.h>		/* Solaris bzero/bcopy/etc... */
#endif

#ifdef USE_PERROR_INSTEAD
#define err(x,y)	{ perror(y); exit(x); }
#else
#include <err.h>
#endif

#ifndef HAVE_SNPRINTF
# ifdef __GNUC__
#  define snprintf(a, b, c...)		sprintf(a, ## c)
# else
#  define snprintf(a, b, c, d)		sprintf(a, c, d)
# endif /* __GNUC__ */
#endif /* !HAVE_SNPRINTF */

#define BUF_SIZ		4096		/* General string buffer size */
#define PAK_SIZ		32768
#define Double		double		/* FP accuracy */

	/* Of the "LINK" statistic, types of link layers */
enum link_type	{ NONE, ETHERNET_IP, ETHERNET_ARP, UNKNOWN_IP };

	/* in get_tcpdump_data, flags on how to operate */
#define GET_TCPD_COUNT_LINKSIZE		0x01
#define GET_TCPD_DO_LIVE		0x02
#define GET_TCPD_DO_LIVE_PROMISC	0x04

typedef struct ip_packet {
	struct ip	hdr;
	union {
		struct tcphdr	tcphdr;
		struct udphdr	udphdr;
		struct icmp	icmp;
	} body;
} ip_packet;

typedef struct packet_data {
	struct timeval		timestamp;
	u_int			packet_len;
	u_char			link_type;
	struct ether_header	ether;
	union {
		struct arphdr		arp;
		struct ip_packet	ip;
		u_char			raw[PAK_SIZ];
	} data;
} packet_data;

/************** Prototypes **************/

/* process.c protos */
void process_tcpdump_header(char *l, packet_data *p, int count_ether_size);
void get_tcpdump_data(char *fname, char *filter, int flags,
        Double capture_seconds, void (*hook)(packet_data *, void **),
	void **args);

/* print_packet.c protos */
void print_packet(packet_data *p);
void print_arp_header(struct arphdr *arp);
void print_ip_header(struct ip *ip);
void print_tcp_header(struct tcphdr *tcp);
void print_udp_header(struct udphdr *udp);
void print_icmp_header(struct icmp *icmp);

/* utils.c protos */
int is_ethernetarp_packet(packet_data *p);
int is_ip_packet(packet_data *p);
unsigned char get_ip_proto(packet_data *p);
int is_ip_tcp_packet(packet_data *p);
int is_ip_udp_packet(packet_data *p);
int is_ip_icmp_packet(packet_data *p);
char *my_basename(char *p);
char *my_get_port(u_short port, int n_only);
char *my_get_proto(u_short proto, int n_only);
char *my_safe_strcpy(char **dst, const char *src);
void my_alarm(Double seconds);
#endif TCPSTAT_TCPSTAT_H

/**************** END *****************/
